#####################
## model Selection ##
#####################

source("polyCWM.R")

MS <- function(
  Y,               # n-dimensional response vector
  X,               # n-dimensional covariate vector
  kmin=1,          # minimum number of groups
  kmax=5,          # maximum number of groups
  rmin=1,          # minimum polynomial degree
  rmax=3,          # maximum polynomial degree
  #IC="BIC",        # model selection criterion adopted ("BIC", "AIC", or "ICL")  
  zlab = NULL,   	                # TRUE classification for the labeled data (which are the first nlab observations in X and Y)
  algorithm = "EM",               # algorithm used (options: "EM" or "CEM")
  method = "BFGS",                # optimization method in each EM iteration
  initialization = "random.soft", # initialization procedure: "random.soft", "random.hard", and "manual" 
  startgroups = NULL,             # vector of initial group-labels: it is used only if initialization="manual"
  iter.max = 1000,                # maximum number of iterations in the EM-algorithm
  threshold = 1.0e-04             # stopping rule in the Aitken rule
  )  
{
  call=match.call()
  
#   if(is.null(G)){
#     G=1:3
#   }
#   if(is.null(modelNames)){
#     modelnames=.cwm$modelNames   
#   }
#   if(is.null(method)){
#     method="BIC"
#   }
  
  n <- length(X)
  
  gridk <- kmin:kmax
  gridr <- rmin:rmax
  numk  <- length(gridk)
  numr  <- length(gridr)
  
  IC <- array(0,c(numk,numr,4),dimnames=list(paste(gridk,"groups",sep=" "),paste("degree",gridr,sep=" "),c("loglik","AIC","BIC","ICL")))

  cont <- 0
  pb  <- winProgressBar("Processing Simulations")
  ee  <- proc.time()
  for(i in 1:numk){
    for(j in 1:numr){
      cont <- cont+1
      temp <- PolyCWM(
        Y = Y,  		                        
        X = X,			                        
        k = gridk[i],                              
        r = gridr[j],                              
        zlab = zlab, 		                
        algorithm = algorithm,               
        method = method,                
        initialization = initialization,  
        startgroups = startgroups,             
        iter.max = iter.max,                
        threshold = threshold,            
        loglikplot = FALSE,              
        SE = FALSE                      
      )
      IC[i,j,1] <- temp$loglik
      IC[i,j,2] <- temp$AIC
      IC[i,j,3] <- temp$BIC
      IC[i,j,4] <- temp$ICL
      
      perc <- cont/(numk*numr)
      setWinProgressBar(pb,perc)
      
    }
  }
  close(pb)
  proc.time()-ee

  cat("\n\n")
  cat("# ----------------------- #","\n")
  cat("# Model Selection Results #","\n")
  cat("# ----------------------- #","\n\n")
  
  # --- #
  # AIC #
  # --- #
  
  BestIndAIC <- which.max(IC[,,2])
  BestIndAIC <- arrayInd(BestIndAIC, .dim=c(numk,numr))
  BestAIC    <- IC[BestIndAIC[1],BestIndAIC[2],2]
  cat("Best AIC value of",BestAIC,"obtained for k =",BestIndAIC[1],"group(s) and a polynomial degree of r =",BestIndAIC[2],"\n\n")
  
  # --- #
  # BIC #
  # --- #
  
  BestIndBIC <- which.max(IC[,,3])
  BestIndBIC <- arrayInd(BestIndBIC, .dim=c(numk,numr))
  BestBIC    <- IC[BestIndBIC[1],BestIndBIC[2],3]
  cat("Best BIC value of",BestBIC,"obtained for k =",BestIndBIC[1],"group(s) and a polynomial degree of r =",BestIndBIC[2],"\n\n")

  # --- #
  # ICL #
  # --- #
  
  BestIndICL <- which.max(IC[,,4])
  BestIndICL <- arrayInd(BestIndICL, .dim=c(numk,numr))
  BestICL    <- IC[BestIndICL[1],BestIndICL[2],4]
  cat("Best ICL value of",BestICL,"obtained for k =",BestIndICL[1],"group(s) and a polynomial degree of r =",BestIndICL[2],"\n\n")
  
  return(
    structure(
      list(
      call   = call,
      IC     = IC
      ),              
      class  = "PolyCWMMS"
    )
  )  
  
}
